==> imspy-0.2.34-py3-none-any.whl: imspy/simulation/timsim/jobs/simulate_frame_distributions.py <==
import numpy as np
import pandas as pd
from tqdm import tqdm

from imspy.simulation.utility import get_frames_numba, accumulated_intensity_cdf_numba, get_z_score_for_percentile, \
    python_list_to_json_string, add_uniform_noise


def simulate_frame_distributions(
        peptides: pd.DataFrame,
        frames: pd.DataFrame,
        z_score: float,
        std_rt: float,
        rt_cycle_length: float,
        verbose: bool = False,
        add_noise: bool = False,
        normalize: bool = False
) -> pd.DataFrame:
    """Simulate frame distributions for peptides.

    Args:
        peptides: Peptide DataFrame.
        frames: Frame DataFrame.
        z_score: Z-score.
        std_rt: Standard deviation of retention time.
        rt_cycle_length: Retention time cycle length in seconds.
        verbose: Verbosity.
        add_noise: Add noise.
        normalize: Normalize frame abundance.

    Returns:
        pd.DataFrame: Peptide DataFrame with frame distributions.
    """

    # distribution parameters
    z_score = get_z_score_for_percentile(target_score=z_score)
    frames_np = frames.time.values

    time_dict = dict(zip(frames.frame_id.values, frames.time.values))

    peptide_rt = peptides

    first_occurrence = []
    last_occurrence = []

    total_list_frames = []
    total_list_frame_contributions = []

    if verbose:
        print("Calculating frame distributions...")

    # generate frame_occurrence and frame_abundance columns
    for _, row in tqdm(peptide_rt.iterrows(), total=peptide_rt.shape[0], desc='frame distribution', ncols=100, disable=not verbose):
        frame_occurrence, frame_abundance = [], []

        rt_value = row.retention_time_gru_predictor
        contributing_frames = get_frames_numba(rt_value, frames_np, std_rt, z_score)

        for frame in contributing_frames:
            time = time_dict[frame]
            start = time - rt_cycle_length
            i = accumulated_intensity_cdf_numba(start, time, rt_value, std_rt)
            frame_occurrence.append(frame)
            frame_abundance.append(i)

        first_occurrence.append(frame_occurrence[0])
        last_occurrence.append(frame_occurrence[-1])

        if add_noise:
            noise_level = np.random.uniform(0.0, 2.0)
            frame_abundance = add_uniform_noise(np.array(frame_abundance), noise_level)

        total_list_frames.append(frame_occurrence)

        if normalize:
            frame_abundance = frame_abundance / np.sum(frame_abundance)

        total_list_frame_contributions.append(frame_abundance)

    if verbose:
        print("Serializing frame distributions to json...")

    peptide_rt['frame_occurrence_start'] = first_occurrence
    peptide_rt['frame_occurrence_end'] = last_occurrence
    peptide_rt['frame_occurrence'] = [list(x) for x in total_list_frames]
    peptide_rt['frame_abundance'] = [list(x) for x in total_list_frame_contributions]

    peptide_rt['frame_occurrence'] = peptide_rt['frame_occurrence'].apply(
        lambda x: python_list_to_json_string(x, as_float=False))

    peptide_rt['frame_abundance'] = peptide_rt['frame_abundance'].apply(python_list_to_json_string)
    peptide_rt = peptides.sort_values(by=['frame_occurrence_start', 'frame_occurrence_end'])

    return peptide_rt

==> imspy-0.3.4-py3-none-any.whl: imspy/simulation/timsim/jobs/simulate_frame_distributions.py <==
import numpy as np
import pandas as pd
from tqdm import tqdm

from imspy.simulation.utility import get_frames_numba, accumulated_intensity_cdf_numba, get_z_score_for_percentile, \
    python_list_to_json_string, add_uniform_noise


def simulate_frame_distributions(
        peptides: pd.DataFrame,
        frames: pd.DataFrame,
        z_score: float,
        std_rt: float,
        rt_cycle_length: float,
        verbose: bool = False,
        add_noise: bool = False,
        normalize: bool = False
) -> pd.DataFrame:
    """Simulate frame distributions for peptides.

    Args:
        peptides: Peptide DataFrame.
        frames: Frame DataFrame.
        z_score: Z-score.
        std_rt: Standard deviation of retention time.
        rt_cycle_length: Retention time cycle length in seconds.
        verbose: Verbosity.
        add_noise: Add noise.
        normalize: Normalize frame abundance.

    Returns:
        pd.DataFrame: Peptide DataFrame with frame distributions.
    """

    # distribution parameters
    z_score = get_z_score_for_percentile(target_score=z_score)
    frames_np = frames.time.values

    time_dict = dict(zip(frames.frame_id.values, frames.time.values))

    peptide_rt = peptides

    first_occurrence = []
    last_occurrence = []

    total_list_frames = []
    total_list_frame_contributions = []

    if verbose:
        print("Calculating frame distributions...")

    # generate frame_occurrence and frame_abundance columns
    for _, row in tqdm(peptide_rt.iterrows(), total=peptide_rt.shape[0], desc='frame distribution', ncols=100, disable=not verbose):
        frame_occurrence, frame_abundance = [], []

        rt_value = row.retention_time_gru_predictor
        contributing_frames = get_frames_numba(rt_value, frames_np, std_rt, z_score)

        for frame in contributing_frames:
            time = time_dict[frame]
            start = time - rt_cycle_length
            i = accumulated_intensity_cdf_numba(start, time, rt_value, std_rt)
            frame_occurrence.append(frame)
            frame_abundance.append(i)

        first_occurrence.append(frame_occurrence[0])
        last_occurrence.append(frame_occurrence[-1])

        if add_noise:
            noise_level = np.random.uniform(0.0, 2.0)
            frame_abundance = add_uniform_noise(np.array(frame_abundance), noise_level)

        total_list_frames.append(frame_occurrence)

        if normalize:
            frame_abundance = frame_abundance / np.sum(frame_abundance)

        total_list_frame_contributions.append(frame_abundance)

    if verbose:
        print("Serializing frame distributions to json...")

    peptide_rt['frame_occurrence_start'] = first_occurrence
    peptide_rt['frame_occurrence_end'] = last_occurrence
    peptide_rt['frame_occurrence'] = [list(x) for x in total_list_frames]
    peptide_rt['frame_abundance'] = [list(x) for x in total_list_frame_contributions]

    # remove entries where frame_abundance is empty
    peptide_rt = peptide_rt[peptide_rt['frame_abundance'].apply(lambda l: len(l) > 0)]

    peptide_rt['frame_occurrence'] = peptide_rt['frame_occurrence'].apply(lambda x: python_list_to_json_string(x, as_float=False))
    peptide_rt['frame_abundance'] = peptide_rt['frame_abundance'].apply(python_list_to_json_string)

    peptide_rt = peptides.sort_values(by=['frame_occurrence_start', 'frame_occurrence_end'])

    return peptide_rt
